
AC_DEFUN([CHECK_OPROFILE_VERSION],
[

	dnl We try to extract the OP_DATADIR macro from the config.h file
	AC_MSG_CHECKING([Oprofile version])

	OP_VERSION=`$OPBIN/opcontrol --version | awk -F" " '{print \[$]3}'`
	AC_MSG_RESULT($OP_VERSION)			

	dnl This is ugly. We were trying to do
	dnl 	echo $OP_VERSION | sed -e 's/\(^[0-9]*.[0-9]*.[0-9]*\).*/\1/'
	dnl but m4 required us to use quadrigraphs 
	OP_VERSION_BASE=`echo $OP_VERSION | \
		sed 's/\(^@<:@0-9@:>@*.@<:@0-9@:>@*.@<:@0-9@:>@*\).*/\1/'`

	if test "$OP_VERSION_BASE" != "0.9.3" ; then
	if test "$OP_VERSION_BASE" != "0.9.4" ; then
	if test "$OP_VERSION_BASE" != "0.9.5" ; then
		AC_MSG_ERROR([Oprofile version $OP_VERSION is not supported.])
	fi fi fi

	dnl Convert version number to a value comparable by preprocessor 
	AC_DEFINE_UNQUOTED(OP_VERSION_BASE, 0x`echo $OP_VERSION_BASE | sed 's/\./0/g'`, [Oprofile version])
])

AC_DEFUN([CHECK_OPROFILE_CONFIG],
[
	dnl Check if the Oprofile is already configured
	AC_MSG_CHECKING([Oprofile $1/config.h])
	if test ! -f "$1/config.h" ; then
		AC_MSG_RESULT(no)
		AC_MSG_ERROR([Please configure Oprofile in $OPDIR])
	else
		AC_MSG_RESULT(yes)
	fi

        dnl We try to extract the OP_DATADIR macro from the config.h file
        AC_MSG_CHECKING([Oprofile version])
        
        OP_VERSION=`fgrep define\ VERSION $1/config.h 2> /dev/null | \
                awk '{ sub( /#define VERSION / , "" ); gsub( /"/ , "" ); print }'`
        AC_MSG_RESULT($OP_VERSION)                      

        dnl This is ugly. We were trying to do
        dnl     echo $OP_VERSION | sed -e 's/\(^[0-9]*.[0-9]*.[0-9]*\).*/\1/'
        dnl but m4 required us to use quadrigraphs 
        OP_VERSION_BASE=`echo $OP_VERSION | \
                sed 's/\(^@<:@0-9@:>@*.@<:@0-9@:>@*.@<:@0-9@:>@*\).*/\1/'`

        if test "$OP_VERSION_BASE" != "0.9.3" ; then
        if test "$OP_VERSION_BASE" != "0.9.4" ; then
        if test "$OP_VERSION_BASE" != "0.9.5" ; then
                AC_MSG_ERROR([Oprofile version $OP_VERSION is not supported.])
        fi fi fi

        dnl Convert version number to a value comparable by preprocessor 
        AC_DEFINE_UNQUOTED(OP_VERSION_BASE, 0x`echo $OP_VERSION_BASE | sed 's/\./0/g'`, [Oprofile version])

	
	dnl We try to extract the OP_DATADIR macro from the config.h file
	OP_DATADIR=`fgrep OP_DATADIR $1/config.h 2> /dev/null | \
		awk '{ sub( /#define OP_DATADIR / , "" ); gsub( /"/ , "" ); print }'`
	AC_DEFINE_UNQUOTED(OP_DATADIR, "$OP_DATADIR", [Oprofile data directory])

	dnl We try to extract the OP_BINDIR macro from the config.h file
	OP_BINDIR=`fgrep OP_BINDIR $1/config.h 2> /dev/null | \
		awk '{ sub( /#define OP_BINDIR / , "" ); gsub( /"/ , "" ); print }'`
	AC_DEFINE_UNQUOTED(OP_BINDIR, "$OP_BINDIR", [Oprofile bin directory])

	dnl Check if oprofiled supports option "--ibs-op-dispatch-op"
	OPD_NO_DISPATCH_OP_OPTION=`fgrep ibs-op-dispatch-op $1/utils/opcontrol 2> /dev/null > /dev/null; echo $?`
	AC_DEFINE_UNQUOTED(OPD_NO_DISPATCH_OP_OPTION, $OPD_NO_DISPATCH_OP_OPTION, [Oprofile daemon does not support IBS-Op dispatched-op option])

	dnl Check if oprofiled supports option "--ibs-op-dcmissinfo-enable"
	OPD_NO_DCMISSINFO_OPTION=`fgrep ibs-op-dcmissinfo-enable $1/utils/opcontrol 2> /dev/null > /dev/null; echo $?`
	AC_DEFINE_UNQUOTED(OPD_NO_DCMISSINFO_OPTION, $OPD_NO_DCMISSINFO_OPTION, [Oprofile daemon does not support IBS-Op DC Miss Info option])
])

