#include <string>
#include <iostream>
#include <map>
#include <dlfcn.h>

#include "qdir.h"
#include "qstring.h"

#include "catranslate_console.cpp"
#include "ca_report.h"
#include "ca_opflat_options.h"
#include "eventsfile.h"
#include "opdata_handler.h"
#include "output_flat_file.h"
#include "catranslate_display.h"
#include "catranslate_console_display.h"


using namespace std;

typedef map<string, op_event_property> OP_EVENT_PROPERTY_MAP;

namespace  {

CATranslate * translate = NULL;


} // end of anon namespace

namespace report {
void * opdata_handle;
create_opdata_t * create_opdata;
delete_opdata_t * delete_opdata;

void * ca_output_handle; 
create_ca_output_t * create_ca_output;
delete_ca_output_t * delete_ca_output;

bool load_opdata_plugin()
{
    string so_name;

    if (ca_options::opdata_plugin_version == "072")
        so_name = "libopdata072.so";
    else if (ca_options::opdata_plugin_version == "081")
        so_name = "libopdata081.so";
    else {
        cerr << "Incorrect opdata plug-in version" << endl;
        return false;
    }

    opdata_handle = dlopen(so_name.c_str(), RTLD_LAZY);
    if (NULL == opdata_handle) {
        cerr << "Failed to load " << so_name << ": " << dlerror() << endl;
        return false;
    }

    create_opdata =
        (create_opdata_t *)dlsym(opdata_handle, "create_opdata");
    delete_opdata = 
        (delete_opdata_t *)dlsym(opdata_handle, "delete_opdata");     
    if (NULL == delete_opdata || NULL == create_opdata) {
        cerr << "Failed to load symbols: " << dlerror() << endl;
        return false;
    }
    return true;
}

bool load_output_plugin()
{
    string so_name;

    if (ca_options::output_plugin == "tbp")
        so_name = "lib_tbp_output.so";
    else if (ca_options::output_plugin == "flat")
        so_name = "lib_flat_output.so";
    else {
        cerr << "Incorrect output plug-ing" << endl;
    }

    ca_output_handle = dlopen(so_name.c_str(), RTLD_LAZY);
    if (NULL == ca_output_handle) {
        cerr << "Failed to load output: " <<  dlerror() << endl;
        return false;
    }

    create_ca_output =
        (create_ca_output_t *)dlsym(ca_output_handle, "create_ca_output");
    delete_ca_output = 
        (delete_ca_output_t *)dlsym(ca_output_handle, "delete_ca_output");     
    if (NULL == delete_ca_output || NULL == create_ca_output) {
        cerr << "Failed to load symbols: " << dlerror() << endl;
        return false;
    }
    return true;
}

bool load_plugins()
{
    bool ret;
    do {
        if (!(ret = load_opdata_plugin()))
            break;

        if (!(ret = load_output_plugin()))
            break;
    } while(0);
    return ret;
}

bool create_objects(CATranslate ** translate, 
    opdata_handler ** m_opdata_handler,
    output ** m_ca_output,
    ca_translate_display ** m_ca_display )
{
    if (!load_plugins())
        return false;

    *m_opdata_handler = create_opdata();
    *m_ca_output = create_ca_output();
    *translate = new ca_translate_console();
    *m_ca_display = new ca_translate_console_display();

    if (NULL == translate || NULL == m_opdata_handler
        || NULL == m_ca_output || NULL == m_ca_display) {
        cerr << "Out of Memory" << endl;
        return false;
    }

    (*m_ca_output)->set_output_path(ca_options::flat_file_path);
    (*m_ca_output)->set_threshold(ca_options::thresholdval);
    (*m_opdata_handler)->set_display(*m_ca_display);

    (*translate)->set_datahandler(*m_opdata_handler);
    (*translate)->set_output(*m_ca_output);
    return true;
}


int XpFileContains(const char * file, const char * str)
{
    if ((NULL == file) || (NULL == str))
	return 0;
    QFile devices (file);
    int nTimes = 0;

    if (devices.open(IO_ReadOnly)) {
	    QTextStream stream(&devices);
	    while (!stream.atEnd()) {
	        QString tmp = stream.readLine();
	        if (tmp.contains (str)) 
		        nTimes++;
	    }
    }
    return nTimes;
}


void import_oprofile_samples()
{
    CATranslate * translate = NULL; 
    opdata_handler * m_opdata_handler = NULL;
    output * m_ca_output = NULL;
    ca_translate_display * m_ca_display = NULL;

    QDir d (ca_options::oprofile_sample_path);

    OP_EVENT_PROPERTY_MAP events_name_map;
    CEventsFile eventfile;
    CpuEvent cpu_event;
    op_event_property op_ebs_properties[MAX_EVENTNUM];
    unsigned int activeCounterMask = 0;

    if (!findOprofileEvents (d, events_name_map))
		printf("No Oprofile sample are found at %s\n"
			"Please try \"ca_opcontrol --dump \"\n" 
			"or start/restart a sampling session\n",
			ca_options::oprofile_sample_path.data());

    OP_EVENT_PROPERTY_MAP::iterator it = events_name_map.begin();
    OP_EVENT_PROPERTY_MAP::iterator end = events_name_map.end();

    vector < string > events_name;
    for (int i = 0; it != end; it++, i++) {
        string name = it->first.data ();
            
        eventfile.findEventByOPName (name.data(), cpu_event);
        name = "event:" + name;
        cout << "import_oprofile_samples " << name << endl;
        events_name.push_back (name);
        op_ebs_properties[i] = it->second;
        op_ebs_properties[i].eventmask = cpu_event.value;
        activeCounterMask |= 1 << i;
    }

    QString q_prd_path = ca_options::prd_path.c_str();
    QString q_sample_path = ca_options::oprofile_sample_path.c_str();


    if (!create_objects(&translate, &m_opdata_handler, 
        &m_ca_output, &m_ca_display)) {
        cerr << "Failed to create objects" << endl;
        return;
    }

    if (NULL == translate || NULL == m_opdata_handler
        || NULL == m_ca_output || NULL == m_ca_display) {
        cerr << "Out of memory" << endl;     
        return;
    }
    
    QString jnc_path = "/var/lib/ca_oprofile/Java";
	QString caSession = "/tmp";
    translate->init(events_name,
            op_ebs_properties,
            q_sample_path, activeCounterMask,
            q_prd_path,
            XpFileContains("/proc/cpuinfo", "processor"),
            jnc_path,
			caSession);

    translate->translate_op_to_ca();
}

void import_prd_file()
{
}

} //end of namespace report

enum OP_FILE_NAME_FIELD
{
    op_field_name = 0,
    op_field_count,
    op_field_unitmask,
};


QString parseOPSampleFileName (const QString & fname)
{
    QStringList strlist = QStringList::split (".", fname);

    if (0 != strlist.size ()) {
        return strlist[op_field_name];
    } else
    return QString ("");
}


void parseOPSampleFileName (const QString & fname,
    OP_EVENT_PROPERTY_MAP &events_name_map)
{
    bool ok;

    QStringList strlist = QStringList::split (".", fname);

    if (0 != strlist.size ()) {
        string parsed_name = string (strlist[op_field_name].data ());
        events_name_map[parsed_name].count =
            strlist[op_field_count].toUInt (&ok);
        events_name_map[parsed_name].unitmask =
            strlist[op_field_unitmask].toUInt (&ok);
        events_name_map[parsed_name].os = true;
        events_name_map[parsed_name].usr = true;
        events_name_map[parsed_name].op_name = parsed_name.c_str ();
    }
}


/**
 * Given a directory, parse the file names in the directory
 * to find the events Oprofile profiled.
 */
bool findOprofileEvents (const QDir & _d, OP_EVENT_PROPERTY_MAP &events_name_map)
{
    QDir d = _d;

    QStringList entries = d.entryList ();
    for (QStringList::Iterator it = entries.begin ();
    it != entries.end (); ++it) {
        if (*it == "." || *it == "..")
            continue;
        QFileInfo info (d, *it);
        if (info.isDir ()) {
            if (!findOprofileEvents (QDir (info.filePath ()), events_name_map))
                return false;
        }
        else {
            parseOPSampleFileName (info.fileName (), events_name_map);
        }
    }

    QString name = d.dirName ();

    if (!d.cdUp ())
        return false;

    return true;
}


