/*
 *   (c) 2003 Advanced Micro Devices, Inc.
 *  YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
 *  AND CONDITIONS OF THE GNU GENERAL PUBLIC
 *  LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
 *  INCLUDED WITH THIS FILE AND POSTED AT
 *  <http://www.gnu.org/licenses/gpl.html>
 *  Support: codeanalyst@amd.com
 *
 * CodeAnalyst example
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#define ARRAY_MAX 100

Display *     display;
int           screen_num;
int *nInt_Array = NULL;			// INTEGER
float *fEmpty_Array = NULL;		// FLOAT
unsigned int *nUintArray = NULL;	// UNSIGNED INT

void ConvertUint(float *fData, unsigned int *iData, int count);
void ConvertInt(float *fData,  int *iData, int count);
unsigned long RunTest(void);

 
int main( int argc, char * argv[] ) 
{
	Window		win;
	int		x, y;
	unsigned int 	width, height;
	unsigned int 	border_width;
	char 		*caption = "Example";

	char 		*displayName = NULL;
    	unsigned int 	dispWidth, dispHeight;
    
	XSizeHints 	*size_hints;
	XWMHints	*wm_hints;
	XClassHint	*class_hints;
	XTextProperty	windowName, iconName;
	//XEvent		report;
	XFontStruct	*font_info;
	XGCValues	values;
	GC		gc;
    
	//  Allocate memory for our structures 
	if ( !( size_hints  = XAllocSizeHints()) || 
		!( wm_hints    = XAllocWMHints()) ||
		!( class_hints = XAllocClassHint())) {
		fprintf(stderr, 
		"%s: failed to allocate memory.\n", argv[0]);
		exit(EXIT_FAILURE);
    	}


	//  Connect to X server 
	if ((display = XOpenDisplay(displayName)) == NULL ) {
		fprintf(stderr, "%s: failed connect to X server %s\n",
			argv[0], displayName);
		exit(EXIT_FAILURE);
	}


	//  Get screen size from display structure macro
	screen_num = DefaultScreen(display);
	dispWidth = DisplayWidth(display, screen_num);
	dispHeight = DisplayHeight(display, screen_num);

	//  Set initial window size and position, and create it 
	x = y = 0;
	width  = dispWidth / 3;
	height = dispWidth / 3;

	win = XCreateSimpleWindow(display, 
		RootWindow(display, screen_num),
		x, y, width, height, border_width,
		BlackPixel(display, screen_num),
		WhitePixel(display, screen_num));


	//  Set hints for window manager before mapping window 
	if (XStringListToTextProperty(&caption, 1, &windowName) == 0) {
		fprintf(stderr, 
			"%s: failed to allocate for windowName.\n",
			argv[0]);
		exit(EXIT_FAILURE);
	}

	if ( XStringListToTextProperty(&caption, 1, &iconName) == 0 ) {
		fprintf(stderr, 
			"%s: failed to allocate for iconName.\n",
			argv[0]);
		exit(EXIT_FAILURE);
	}

	size_hints->flags = PPosition | PSize | PMinSize;
	size_hints->min_width = 200;
	size_hints->min_height = 100;

	wm_hints->flags         = StateHint | InputHint;
	wm_hints->initial_state = NormalState;
	wm_hints->input         = True;

	class_hints->res_name   = argv[0];
	class_hints->res_class  = "hellox";

	XSetWMProperties(display, win, &windowName, &iconName, argv, 
			argc, size_hints, wm_hints, class_hints);


	//  Choose which events we want to handle
	XSelectInput(display, win, ExposureMask | KeyPressMask |
		ButtonPressMask | StructureNotifyMask);


	// Load a font called "9x15" 
	if ( (font_info = XLoadQueryFont(display, "9x15")) == NULL ) {
		fprintf(stderr, 
			"%s: cannot open 9x15 font.\n", argv[0]);
		exit(EXIT_FAILURE);
	}


	//  Create graphics context 
	gc = XCreateGC(display, win, 0, &values);

	XSetFont(display, gc, font_info->fid);
	XSetForeground(display, gc, BlackPixel(display, screen_num));

	//  Display Window
	XMapWindow(display, win);

	unsigned long startTicks = 0;	// START TICKS
	unsigned long endTicks = 0;	// END TICKS
	
	// TIME IT TAKES TO CALL TimeStamp AND RETURN
	unsigned long latentcy = 0;	
    
	// GET TIME STAMP LATENTCY
	for (int i=0; i<15; i++) {		// 15 TIMES TO MAKE SURE
		__asm__ __volatile__ ( 
		"xorl %%eax, %%eax\n\t"
		"xorl %%ebx, %%ebx\n\t"
		"xorl %%ecx, %%ecx\n\t"
		"xorl %%edx, %%edx\n\t"
		"cpuid\n\t"
		"rdtsc\n\t"
		:"=a"(startTicks)
      		);
		
		__asm__ __volatile__ ( 
		"xorl %%eax, %%eax\n\t"
		"xorl %%ebx, %%ebx\n\t"
		"xorl %%ecx, %%ecx\n\t"
		"xorl %%edx, %%edx\n\t"
		"cpuid\n\t"
		"rdtsc\n\t"
		:"=a"(endTicks)
      		);
	}

	latentcy = endTicks - startTicks;

	
	//  ARRAYS ALIGNED ON 64 BYTE BOUNDRY
	nUintArray = (unsigned int *)((((unsigned int) malloc(
		sizeof (unsigned int)*(ARRAY_MAX)+63))+63) & ~63);

	nInt_Array = (int *)((((int) malloc (
		sizeof (int)*(ARRAY_MAX)+63))+63) & ~63);
	fEmpty_Array = (float *)((((int) malloc (
		sizeof (float)*(ARRAY_MAX)+63))+63) & ~63);

	for(int i=0; i<ARRAY_MAX;i++)
	{
		nInt_Array[i] = rand();
		nUintArray[i] = (unsigned int)(rand());
	}

	if (!nInt_Array || !nUintArray || !fEmpty_Array)
	{
		fprintf(stderr," -- failed to allocate arraies.\n");
		if (nInt_Array)
			free(nInt_Array);

		if (nUintArray)
			free(nUintArray);
		if (fEmpty_Array)
			free(fEmpty_Array);

		exit(EXIT_FAILURE);
	}

	static char message[256];
	static int length;
	static int font_height;
	static int msg_x, msg_y;
	length = XTextWidth(font_info, message, strlen(message));
	msg_x  = (width - length) / 2;
	font_height = font_info->ascent + font_info->descent;
	msg_y  = (height + font_height) / 2;

	while ( 1 ) {
		length = XTextWidth(font_info, 
				message, strlen(message));
		msg_x  = (width - length) / 2;

		font_height = font_info->ascent + 
				font_info->descent;
		msg_y  = (height + font_height) / 2;

		unsigned long deltaTicks = RunTest() - latentcy;
		sprintf(message, "ticks = %10u", deltaTicks);
		XClearWindow(display, win);		
		XDrawString(display, win, gc, msg_x, msg_y,
				message, strlen(message));
	}

	return EXIT_SUCCESS;   /*  We shouldn't get here  */
}


/******************************************************************
 * ConvertUint
 * CONVERT UNSIGNED INTEGER TO FLOAT
 ******************************************************************
*/
void ConvertUint(float *fData, unsigned int *iData, int count)
{
	float	fTemp=0.0f;

	for(int i=0;i<count;i++) {
		static const float fScale = 0.750f;
		fTemp = iData[i];	
		fTemp *= fScale;
		fData[i] = fTemp;
	}
}

/*****************************************************************
 * ConvertInt
 * CONVERT SIGNED INTEGER TO FLOAT
 *****************************************************************/
void ConvertInt(float *fData,  int *iData, int count)
{
	float	fTemp=0.0f;

	for(int i=0;i<count;i++) {
		static const float fScale = 0.750f;
		fTemp = iData[i];
		fTemp *= fScale;
		fData[i] = fTemp;
	}
}

unsigned long RunTest(void)
{
	unsigned long tick1, tick2;

	ConvertInt(&fEmpty_Array[0],&nInt_Array[0], ARRAY_MAX);
//	ConvertInt(&fEmpty_Array[0],&nInt_Array[0],ARRAY_MAX);
	__asm__ __volatile__ ( 
		"xorl %%eax, %%eax\n\t"
		"xorl %%ebx, %%ebx\n\t"
		"xorl %%ecx, %%ecx\n\t"
		"xorl %%edx, %%edx\n\t"
		"cpuid\n\t"
		"rdtsc\n\t"
		:"=a"(tick1)
      		);
		
	ConvertInt(&fEmpty_Array[0],&nInt_Array[0], ARRAY_MAX);
//	ConvertInt(&fEmpty_Array[0],&nInt_Array[0],ARRAY_MAX);

	__asm__ __volatile__ ( 
		"xorl %%eax, %%eax\n\t"
		"xorl %%ebx, %%ebx\n\t"
		"xorl %%ecx, %%ecx\n\t"
		"xorl %%edx, %%edx\n\t"
		"cpuid\n\t"
		"rdtsc\n\t"
		:"=a"(tick2)
      		);

	return (tick2 - tick1);
}

