//$Id: optionsdlg.cpp,v 1.4 2006/05/15 22:09:22 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qbuttongroup.h>
#include <qlayout.h> 

#include "stdafx.h"
#include "optionsdlg.h"
#include "auxil.h"
#include "atuneoptions.h"
#include "xp.h"
#include "application.h"


// declared in auxil.cpp
extern SymbolMap g_SymMap;

#define	JVMTI_STRING "-agentpath:/usr/local/AMD/CodeAnalyst/bin/libCAJVMTIA.so"

const char K8_EVENT_FORMAT[] = "%2u %2u %[^n]";

OptionsDlg::OptionsDlg ( bool new_file, ATuneFile * caw_file, QWidget * parent,
						const char * name, bool modal, bool warning, 
						WFlags fl )
						: IOptionsDlg ( parent, name, modal, fl )
{
	CATuneOptions ao;
	QString path = "";
	DWORD jpa = 0;

	// connect events
	QObject::connect ( m_timer_trigger, SIGNAL (clicked()), 
		SLOT (onTriggerChanged()) );
	QObject::connect ( m_event_trigger, SIGNAL (clicked()),
		SLOT (onTriggerChanged()) );
	QObject::connect ( m_program, SIGNAL (textChanged (const QString&)),
		SLOT (onProgramChanged (const QString&)) );
	QObject::connect ( m_stop_on_app, SIGNAL (stateChanged( int )),
		SLOT (onStopAppChanged()) );
	QObject::connect ( m_no_duration,SIGNAL (stateChanged( int )),
		SLOT (onNoDuration()) );
	QObject::connect (m_sort_index, SIGNAL (clicked ()), SLOT (onSortEvents ()));
	QObject::connect (m_sort_name, SIGNAL (clicked ()), SLOT (onSortEvents ()));
	QObject::connect(chkJava, SIGNAL (clicked()), SLOT(onIncludeJava()));

	//finish building the events
	createEvents();

	// Initialize our validator objects
	m_val_interval	= new QDoubleValidator ( 0.0001, 100000, 4, this );
	m_val_duration	= new QIntValidator ( 0, 36000,  this );
	m_val_delay		= new QIntValidator ( 0, 3600,   this );
	m_new		= new_file;
	m_show_apic_warning = warning;
	m_caw_file		= caw_file;
	eventIndexArray = NULL;

	// Determine what Cpu
	detectCpu();

	//display an informative message if we are running on a box without an APIC
	// which cannot do any sampling
	if ((! ebpAllowed()) && (m_show_apic_warning)) {
		QMessageBox::information( 0, "Warning",
			"There is no on-chip local apic.\n"
			"The processor you are running on does not "
			"have the\ncapabilities to perform Event "
			"Based Profiling.\n" );
	}

	if ( !ebpAllowed() ) {
		m_event_trigger->setEnabled ( FALSE );
		enableTimerObjects ( TRUE  );
		enableEventObjects ( FALSE );
	}

	if ( m_new ) {
		// if this is a new file, default to timer-based sampling
		enableTimerObjects ( TRUE  );
		enableEventObjects ( FALSE );
		m_project_name->setReadOnly (FALSE);
		m_proj_path->setReadOnly (FALSE);
	}

	// Initialize the project directory
	ao.getDefaultProjectDir (path);
	m_default_project_dir = path;

	m_proj_path->setText ( m_default_project_dir );

	ao.getJPA(&jpa);
	if (jpa == JVMTI) {
		rdo_jvmti->setChecked(true);
		rdo_jvmpi->setChecked(false);
	} else {
		rdo_jvmti->setChecked(false);
		rdo_jvmpi->setChecked(true);
	}



} //OptionsDlg::OptionsDlg


OptionsDlg::~OptionsDlg()
{
	// no need to delete child widgets, Qt does it all for us
	if (NULL != eventIndexArray)
		delete [] eventIndexArray;
}


void OptionsDlg::onApply()
{
	if( !validate() ) return;
	saveChanges();   
}


void OptionsDlg::onOk()
{
	if( !validate() ) return;

	if ( saveChanges()) {
		accept();
	}
	return;
}


void OptionsDlg::onProgramChanged ( const QString& txt )
{
	if( txt.isEmpty() ) {
		m_stop_on_app->setEnabled ( FALSE );
		m_stop_on_app->setChecked ( FALSE );
		m_break_after->setEnabled ( FALSE );
		m_break_after->setChecked ( FALSE );
		m_no_duration->setChecked ( FALSE );
		m_no_duration->setEnabled ( FALSE );
	} else {
		m_stop_on_app->setEnabled ( TRUE );
		m_break_after->setEnabled ( !m_no_duration->isChecked() );
		m_no_duration->setEnabled (m_stop_on_app->isChecked());
	}
}


void OptionsDlg::onBrowseDir()
{
	QCString path;

	path = AuxGetDirectoryDialog ( "Enter project directory", 
		(char *) m_default_project_dir.data(),
		this);

	// Make sure the project directory format is valid
	if ( ! path.isEmpty() ) m_proj_path->setText ( path );
}


void OptionsDlg::onBrowseWorkDir()
{
	QCString path;

	path = AuxGetDirectoryDialog ("Enter working directory", 
		(char *) m_default_project_dir.data(),
		this);

	// Make sure the project directory format is valid
	if ( ! path.isEmpty() ) m_work_dir->setText ( path );
}


void OptionsDlg::onBrowsePrograms()
{
	QString prog = QFileDialog::getOpenFileName ( NULL, "Any file (*)", 
		this, "FileBrowse", "Select "
		"an application to launch" );

	m_stop_on_app->setEnabled ( ! prog.isEmpty() );
	if ( prog.isEmpty() ) return;

	m_program->setText( "\"" + prog + "\"" );
}


bool OptionsDlg::saveChanges()
{
	TBS_SESSION_OPTIONS tbp;
	GENERAL_OPTIONS     gen;
	EBS_SESSION_OPTIONS ebp;
	CATuneOptions ao;
	uint sort_by_index = SORT_BY_INDEX;

	if (!m_sort_index->isChecked())
		sort_by_index = NO_SORT_BY_INDEX;
	ao.setSortEventByIndex (sort_by_index);

	if (rdo_jvmpi->isChecked())
		ao.setJPA(JVMPI);
	else
		ao.setJPA(JVMTI);


	if ( m_new ) {
		ebp.bStopOnApp = tbp.bStopOnApp = gen.bStopOnApp 
			= m_stop_on_app->isChecked();
		ebp.bBreakAfter = tbp.bBreakAfter = gen.bBreakAfter  
			= m_break_after->isChecked();
		ebp.bJitInfo = tbp.bJitInfo = gen.bJitInfo = chkJava->isChecked();
		tbp.msInterval = m_interval->text().toFloat();
		ebp.ProgramPath = tbp.ProgramPath = gen.ProgramPath  
			= m_program->text();
		gen.Dir = m_proj_path->text();
		ebp.WorkDir = tbp.WorkDir = gen.WorkDir = m_work_dir->text();
		ebp.sDelay = tbp.sDelay = gen.sDelay = m_delay->text().toUInt();
		ebp.sDuration = tbp.sDuration = gen.sDuration
			= m_duration->text().toUInt();
		gen.SessionName  = m_session_name->text();
		tbp.SessionName  = "_TBS_GENERAL_OPTIONS";
		ebp.SessionName  = "_EBS_GENERAL_OPTIONS";
		tbp.NumCpus      = XpGetNumCpus();

		if ( m_timer_trigger->isChecked() ) {
			gen.Trigger = TIMER_TRIGGER;
		} else if( m_event_trigger->isChecked() ) {
			if( m_precise_branch_mispredict->isChecked() )
				gen.Trigger = EVENT_TRIGGER_PBM;
			else
				gen.Trigger = EVENT_TRIGGER;
		} else {
			// not able to do any sampling
			gen.Trigger = NO_TRIGGER;
		}

		getEbpSettings ( &ebp );

		// create the directory for the new project
		QDir dir(m_proj_path->text() + m_project_name->text());
		if (dir.exists())
			ApplicationWindow::folderRemove(dir);
		if (!dir.mkdir (m_proj_path->text() + m_project_name->text())){
			QMessageBox::information (this,"Directory error", "Cannot create "
				"the requested directory at " 
				+ m_proj_path->text() 
				+ m_project_name->text());
			return false;
		}

		// Save the new project
		QString ProjPath = m_proj_path->text() + m_project_name->text()
			+ XPS_PATH_SLASH + m_project_name->text() + ".caw";
		m_caw_file = new ATuneFile ( ProjPath, TRUE );
		if (NULL == m_caw_file) {
			QMessageBox::critical (this, "Memory error", "Not enough memory is"
				" available");
			return false;
		}

		m_caw_file->newAtuneFile( gen, tbp, ebp );

	} else {
		m_caw_file->getGeneralOptions ( &gen );
		m_caw_file->getTbsSessionOptions ( GENERAL_SESSION, &tbp );
		m_caw_file->getEbsSessionOptions ( GENERAL_SESSION, &ebp );

		tbp.msInterval = m_interval->text().toFloat();
		ebp.ProgramPath = tbp.ProgramPath = gen.ProgramPath
			= m_program->text();
		ebp.WorkDir = tbp.WorkDir = gen.WorkDir = m_work_dir->text();
		ebp.bStopOnApp = tbp.bStopOnApp = gen.bStopOnApp
			= m_stop_on_app->isChecked();
		ebp.bBreakAfter = tbp.bBreakAfter = gen.bBreakAfter
			= m_break_after->isChecked();
		ebp.bJitInfo = tbp.bJitInfo = gen.bJitInfo = chkJava->isChecked();
		ebp.sDelay = tbp.sDelay = gen.sDelay = m_delay->text().toUInt();
		ebp.sDuration = tbp.sDuration = gen.sDuration
			= m_duration->text().toUInt();
		gen.SessionName  = m_session_name->text();
		tbp.SessionName  = "_TBS_GENERAL_SESSION_";
		ebp.SessionName  = "_EBS_GENERAL_SESSION_";

		getEbpSettings ( &ebp );

		if ( m_event_trigger->isChecked() ) {
			if ( m_precise_branch_mispredict->isChecked() )
				gen.Trigger = EVENT_TRIGGER_PBM;
			else
				gen.Trigger = EVENT_TRIGGER;
		} else if ( m_timer_trigger->isChecked() ) {
			gen.Trigger = TIMER_TRIGGER;
		}

		m_caw_file->setGeneralOptions ( gen );
		m_caw_file->setTbsSessionOptions ( GENERAL_SESSION, tbp );
		m_caw_file->setEbsSessionOptions ( GENERAL_SESSION, ebp );
		m_caw_file->saveAtuneFile();
	}

	if (EVENT_TRIGGER == gen.Trigger || EVENT_TRIGGER_PBM == gen.Trigger) {
		bool no_events = true;
		for (int i = 0; i < MAX_EVENTNUM; i++)
			if (0 != ebp.comboindex[i]) {
				no_events = false;
				break;
			}
			if (no_events) {
				QMessageBox::critical (this, "Warning", "Event Based "
					"Sampling selected but no valid "
					"events were selcted." );

				// Switch to the event page and hight event selector.
				m_tab_widget->setCurrentPage ( EVENT_PAGE );
				tab_2->setFocus();
				m_combos[0][EVENT_TYPE]->setFocus( );
				return FALSE;
			}
	}
	return TRUE;
} //OptionsDlg::saveChanges


bool OptionsDlg::validate()
{
	QString err_msg;
	QDir dir;
	bool temp;
	int i;

	// Make sure the project direcotry is not empty
	if( m_proj_path->text().isEmpty() ) {
		QMessageBox::information (this, "Invalid Input", 
			"Please select a project directory.");
		m_dir_browse->setFocus();
		return FALSE;
	}

	// make sure the project directory exists
	dir.setPath ( m_proj_path->text() );

	// don't accept relative project directories
	if ( dir.isRelative() ) {
		QMessageBox::information (0, "invalid directory", "The project "
			"directory specified is invalid" );

		m_proj_path->selectAll();
		m_proj_path->setFocus();

		return FALSE;
	}

	if( ! dir.exists() ) {
		int answer = QMessageBox::information (this, "invalid directory", 
			"The directory \"" 
			+ m_proj_path->text() 
			+"\" does not exist.\nCreate?", 
			QMessageBox::Yes, 
			QMessageBox::No );

		if ( QMessageBox::Yes == answer ) {
			dir.mkdir (m_proj_path->text());

			if( !dir.exists (m_proj_path->text()) ) {
				QMessageBox::information (this, "invalid directory", "The "
					"directory specified is invalid." );

				m_proj_path->selectAll();
				m_proj_path->setFocus();
				return FALSE;
			}
		} else if ( QMessageBox::No == answer ) {
			m_proj_path->selectAll();
			m_proj_path->setFocus();

			return FALSE;
		}
	}

	// make sure the project path has an '/' character at the end
	if ( XPS_PATH_SLASH != m_proj_path->text().right (1) )
		m_proj_path->setText ( m_proj_path->text().append ( XPS_PATH_SLASH ) );

	// Make sure the project name field is valid
	if ( m_new && m_project_name->text().isEmpty() ) {
		QMessageBox::information ( this, "Invalid Input", "Please select a "
			"project name." );

		m_project_name->setFocus();
		return FALSE;
	}

	// Make sure there is not already a project with the directory
	if ( m_new && dir.exists ( m_proj_path->text() + m_project_name->text())) {
		if ( QMessageBox::No == QMessageBox::information( this, "Error",
			"A Project already exists in that directory. "
			"Create anyway?", QMessageBox::Yes, QMessageBox::No ) )
			return FALSE;
	}

	// Validate the working directory only if the user has entered some text
	if ( ! m_work_dir->text().isEmpty() ) {
		dir.setPath( m_work_dir->text() );
		// don't accept relative working directories
		if( dir.isRelative() ) {
			QMessageBox::information (NULL, "Invalid directory", "The working "
				"directory specified is invalid" );

			m_work_dir->selectAll();
			m_work_dir->setFocus();
			return FALSE;
		}

		// Make sure the target working directy actually exits
		if (! dir.exists( m_work_dir->text())) {
			int answer = QMessageBox::information (this, "invalid directory", 
				"The directory \"" 
				+ m_work_dir->text() +
				"\" does not exist.\nCreate?",
				QMessageBox::Yes,
				QMessageBox::No );

			if ( QMessageBox::Yes == answer ) {
				dir.mkdir(m_work_dir->text());

				if( !dir.exists(m_proj_path->text()) ) {
					QMessageBox::information (this, "invalid directory", 
						"The directory specified is "
						"invalid." );

					m_work_dir->selectAll();
					m_work_dir->setFocus();
					return FALSE;
				}
			} else if ( QMessageBox::No == answer ) {
				m_work_dir->selectAll();
				m_work_dir->setFocus();
				return FALSE;
			}
		}
	}

	// Make sure we have an acceptable interval
	QString txt = m_interval->text();
	if (QValidator::Invalid ==  m_val_interval->validate ( txt, i ) ) {
		err_msg = "Please enter an acceptable value for 'Interval' "
			"\nAcceptable values are from 1 ms to 100000 ms or 100 seconds.";

		QMessageBox::information ( this, "Invalid input", err_msg );

		m_interval->setFocus ();
		m_interval->selectAll();
		return FALSE;
	}

	// Check if APIC is enabled.
	if( m_interval->isEnabled() && (m_interval->text().toFloat() < 1.0f) 
		&& (! ebpAllowed()) ) {
			QMessageBox::warning (this, "Error", "The interval can not be less "
				"than 1 ms without\non-chip local apic.\n");

			m_interval->setText ( "1" );
			return FALSE;
	} else if( m_interval->isEnabled() && (m_interval->text().toFloat() 
		< 0.09f) && (ebpAllowed()) ) {
			// Compare to 0.09 due to math inaccuracy.  The real minimum is
			//  0.1ms
			QMessageBox::warning(this, "Error", "The interval can not be less "
				"than 0.1 ms with\non-chip local apic.\n");
			m_interval->setText ( "0.1" );
			return FALSE;
	}

	// Make sure we have an acceptable duration
	txt = m_duration->text();
	temp = (QValidator::Acceptable != m_val_duration->validate (txt, i));
	if ((temp ) || ((0 ==txt.toUInt()) && (!m_no_duration->isChecked ()))) {
		err_msg = "Please enter an acceptable value for 'Duration' \n"
			"Acceptable values are from 1 s to 36000 s or 10 hours.";

		QMessageBox::information ( this, "Invalid input", err_msg );

		m_duration->setFocus ();
		m_duration->selectAll();

		return FALSE;
	}

	// Make sure we have an acceptable delay
	txt = m_delay->text();
	if(QValidator::Invalid == m_val_delay->validate ( txt, i ) ) {
		err_msg = "Please enter an acceptable value for 'Delay' \n"
			"Acceptable values are from 0 s to 3600 s or 1 hour.";

		QMessageBox::information ( this, "Invalid input", err_msg );

		m_delay->setFocus ();
		m_delay->selectAll();

		return FALSE;
	}

	txt = m_program->text();
	if( ! txt.isEmpty() ) {
		int first_quote = txt.find ("\"");
		int second_quote = txt.find ("\"", 1);

		if (( -1 == first_quote) || ( -1 == second_quote )) {
			QMessageBox::information( NULL, "Invalid input", "The "
				"application must be in quotes." );
			return FALSE;
		}

		QString app = txt.mid ((first_quote + 1), (second_quote 
			- first_quote - 1) );

		if (!chkJava->isChecked()) {
			if ( ! QFile::exists (app) ) {
				QMessageBox::information (this, "Invalid Input", "The " + app 
					+ " application specified does not "
					"exist." );

				m_program->setFocus();
				m_program->setSelection ( first_quote + 1, second_quote);

				return FALSE;
			}
		} else {
			//java box checked.
			//if executable is java or java.exe
			if ((app.endsWith ("java"))) {
				int agent_start;
				int remove_index;
				QString replace_agent;
				QString remove_string;
				if (rdo_jvmpi->isChecked()) {
					// JVMPI
					replace_agent = "-XrunCAJVMPIA";
					remove_string = JVMTI_STRING;
				} else {
					// JVMTI
					replace_agent = JVMTI_STRING;
					remove_string = "-XrunCAJVMPIA";
				}

				remove_index = txt.find(remove_string, 0, false);
				txt.remove(remove_index, remove_string.length());

				agent_start = txt.find(replace_agent, 0, false);
				if (-1 != agent_start) {
					//it's already in there.
					txt.replace(agent_start, replace_agent.length(), replace_agent);
					m_program->setText(txt);
				} else {
					replace_agent.prepend(" ");
					txt.insert(second_quote + 1, replace_agent);
					m_program->setText(txt);
				}
			}
		}
	}



	//Makes sure events aren't duplicated accross counters
	return validateEvents();
} //OptionsDlg::validate()


QString OptionsDlg::getCpuModelName()
{
	QString cpuModelString = "";
	QFile * file = new QFile ("/proc/cpuinfo");
	if (NULL == file) 
		return cpuModelString ;

	QTextStream stream(file);
	stream.setDevice (file);
	if (!file->open(IO_ReadOnly)) 
		return cpuModelString ;

	while (!stream.atEnd()) {
		QString line = stream.readLine();	
		if (line.contains("model name")) {
			cpuModelString = line;
			break;
		}
	}	

	int beginIndex = cpuModelString.find(":") + 1;	/* remove space */
	cpuModelString = cpuModelString.mid(beginIndex, cpuModelString.length());
	return cpuModelString;
}


void OptionsDlg::detectCpu()
{
	char vendor_id[15];
	unsigned long family;
	unsigned long model;
	unsigned long stepping;
	unsigned long features;
	QString cpu_desc;
	char * dir = getenv("OPROFILE_EVENTS_DIR");


	CpuId ( vendor_id, &family, &model, &stepping, &features );

	
	// what vendor?
	if (0 == strncmp (vendor_id, "AuthenticAMD", 12) ) {
		QString file = helpGetEventFile(family, model);
		loadAMDEvents ( file );
	}
	cpu_desc.append(getCpuModelName());

	// determine if we have a local apic for sampling
	if( ONCHIP_LOCAL_APIC == (features & ONCHIP_LOCAL_APIC) )
		m_has_local_apic = TRUE;
	else 
		m_has_local_apic = FALSE;

	QString numbers;
	cpu_desc.append (numbers);
	m_cpu_desc->setText (cpu_desc);
} //OptionsDlg::detectCpu


void OptionsDlg::loadAMDEvents (const char * event_path)
{
	QString current_path;
	uint max_width = 0;
	int i;
	CATuneOptions ao;
	uint sort_by_index = SORT_BY_INDEX;

	// get the full path to the events file
	XpGetAppPath (current_path);
	current_path = current_path + event_path;

	if ( ! m_events_file.Open (event_path) ) {
		QMessageBox::information ( this, "Error", "Unable to open the "
			"events file ." + QString (event_path) );
	}

	// have an empty entry in each combo box
	for (i = 0; i < MAX_EVENTNUM; i ++) {
		m_combos[i][EVENT_NAME]->insertItem ( "" );
		m_combos[i][EVENT_TYPE]->insertItem ( "" );
	}

	EventList::const_iterator elit;
	for ( elit = m_events_file.FirstEvent(); elit 
		!= m_events_file.EndOfEvents(); elit++ ) {
			uint width;

			for (i = 0; i < MAX_EVENTNUM; i ++) {
				m_combos[i][EVENT_NAME]->insertItem ( elit->name );
				m_combos[i][EVENT_TYPE]->insertItem ( QString::number (elit->value,
					HEX_BASE) );
			}

			width = fontMetrics().width (elit->name);
			if ( width > max_width ) max_width = width;
	}

	eventIndexArray = new int[m_combos[0][EVENT_TYPE]->count()];
	for (i = 0; i < m_combos[0][EVENT_TYPE]->count(); i++)
		eventIndexArray[i] = i;

	for (i = 0; i < MAX_EVENTNUM; i ++) {
		m_combos[i][EVENT_NAME]->listBox()->setMinimumWidth (max_width 
			+ fontMetrics().width ("TT") );
	}

	ao.getSortEventByIndex (&sort_by_index);
	if (SORT_BY_INDEX != sort_by_index)
		m_sort_name->setChecked (true);
} // OptionsDlg::loadAMDEvents


void OptionsDlg::onTriggerChanged()
{
	int i;
	if( m_event_trigger->isChecked() ) {
		enableTimerObjects( FALSE );
		enableEventObjects( TRUE  );

		for (i = 0; i < MAX_EVENTNUM; i ++) {
			onEventChanged (m_combos[i][EVENT_NAME]->currentItem(), i);
		}

		m_profile_group->setEnabled( TRUE );
	} else if( m_timer_trigger->isChecked() ) {
		enableTimerObjects( TRUE  );
		enableEventObjects( FALSE );

		m_profile_group->setEnabled( TRUE );
	}
}

void OptionsDlg::onIncludeJava()
{
	if (chkJava->isChecked())
		bgp_java->setEnabled(TRUE);
	else
		bgp_java->setEnabled(FALSE);
}


void OptionsDlg::onSortEvents ()
{
	ElementComboBox * sort_by = NULL;
	int save_index[MAX_EVENTNUM];
	uint save_masks[MAX_EVENTNUM];
	bool save_chk[MAX_EVENTNUM][2];
	int i;
	for (i = 0; i < MAX_EVENTNUM; i ++) {
		save_index[i] = eventIndexArray [m_combos[i][EVENT_TYPE]->currentItem()];
		save_masks[i] = getEventUnitMask (i);
		save_chk[i][0] = m_checks[i][CHECK_USR]->isChecked();
		save_chk[i][1] = m_checks[i][CHECK_OS]->isChecked();
	}

	if (m_sort_index->isChecked ())
		sort_by = m_combos[0][EVENT_TYPE];
	else
		sort_by = m_combos[0][EVENT_NAME];

	//    setUpdatesEnabled (false);
	emit (setChildUpdateEnable ( false));
	quickSortEvents (sort_by, 1, (sort_by->count() - 1) );
	//    setUpdatesEnabled (true);
	emit (setChildUpdateEnable ( true));
	update();

	for (i = 0; i < MAX_EVENTNUM; i ++) {
		m_combos[i][EVENT_NAME]->setCurrentItem( indexOfEvent (save_index[i]) );
		setEventUnitMask (save_masks[i], i);
		m_checks[i][CHECK_USR]->setChecked(save_chk[i][0]);
		m_checks[i][CHECK_OS]->setChecked(save_chk[i][1]);
	}
}


void OptionsDlg::enableTimerObjects (bool enabled)
{
	m_timer_trigger->setChecked( enabled );
	m_interval->setEnabled( enabled );
}


void OptionsDlg::enableEventObjects (bool enabled)
{
	m_event_trigger->setChecked( enabled );

	for (int i = 0; i < MAX_EVENTNUM; i ++) {
		m_combos[i][EVENT_NAME]->setEnabled( enabled );
		m_combos[i][EVENT_TYPE]->setEnabled( enabled );

		m_lines[i][LINE_COUNT]->setEnabled( enabled );
		m_lines[i][LINE_SOURCE]->setEnabled( enabled );
		m_checks[i][CHECK_OS]->setEnabled( enabled );
		m_checks[i][CHECK_USR]->setEnabled( enabled );
	}	
}


bool OptionsDlg::initialize()
{
	TBS_SESSION_OPTIONS opt;
	GENERAL_OPTIONS     gen;
	EBS_SESSION_OPTIONS evt;
	int i;

	m_caw_file->getEbsSessionOptions( GENERAL_SESSION, &evt );
	m_caw_file->getTbsSessionOptions( GENERAL_SESSION, &opt );
	m_caw_file->getGeneralOptions( &gen );

	setCaption( "Settings for " + m_caw_file->getName() );
	m_project_name->setText( m_caw_file->getName() );
	m_proj_path->setText ( m_caw_file->getDir() );
	if (m_caw_file->getNumTbsSessions() > 1) {
		m_file_name->setText ((const char *) gen.SessionName 
			+QString::number (m_caw_file->getNumTbsSessions()
			- 1) + ".tbp" );
	} else
		m_file_name->setText (gen.SessionName + ".tbp" );

	// Initialize General Settings
	m_session_name->setText( gen.SessionName );
	m_duration->setText( QString::number( gen.sDuration ) );
	m_program->setText( gen.ProgramPath );
	m_work_dir->setText( gen.WorkDir );
	chkJava->setChecked(gen.bJitInfo);
	m_delay->setText( QString::number( gen.sDelay ) );

	m_stop_on_app->setEnabled( ! gen.ProgramPath.isEmpty() );
	m_stop_on_app->setChecked( (!gen.ProgramPath.isEmpty()) && gen.bStopOnApp);
	m_no_duration->setEnabled (m_stop_on_app->isChecked());
	m_no_duration->setChecked (0 == gen.sDuration);

	m_break_after->setEnabled ( (!gen.ProgramPath.isEmpty()) 
		&& (0 != gen.sDuration) );
	m_break_after->setChecked( (! gen.ProgramPath.isEmpty()) 
		&& gen.bBreakAfter);

	// Initialize Timer Settings
	m_interval->setText ( QString::number( opt.msInterval ) );

	emit (setChildUpdateEnable (true));
	// Initialize Event Settings
	for (i = 0; i < MAX_EVENTNUM; i ++) {
		m_combos[i][EVENT_NAME]->setCurrentItem( evt.comboindex[i] );
		if (evt.count[i] > 0)
			m_lines[i][LINE_COUNT]->setText( QString::number (evt.count[i]) );
		else
			m_lines[i][LINE_COUNT]->setText( "100000");
		setEventUnitMask( evt.unitmask[i], i );

		m_checks[i][CHECK_OS]->setChecked( 1 == evt.Os[i] );
		m_checks[i][CHECK_USR]->setChecked( 1 == evt.Usr[i] );
	}

	if( !ebpAllowed() ) {
		m_timer_trigger->setChecked( FALSE );
		m_event_trigger->setEnabled( FALSE );
		enableTimerObjects( TRUE  );
		enableEventObjects( FALSE );
	}

	if( TIMER_TRIGGER == gen.Trigger ) {
		m_timer_trigger->setChecked( TRUE );
		enableTimerObjects  ( TRUE  );
		enableEventObjects  ( FALSE );
	} else if( (EVENT_TRIGGER == gen.Trigger ) && ebpAllowed() ) {
		m_event_trigger->setChecked( TRUE );
		enableTimerObjects  ( FALSE );
		enableEventObjects  ( TRUE  );

		m_precise_branch_mispredict->setEnabled( TRUE );
		m_precise_branch_mispredict->setChecked( FALSE );
	} else if( EVENT_TRIGGER_PBM == gen.Trigger ) {
		m_event_trigger->setChecked( TRUE );
		enableTimerObjects  ( FALSE );
		enableEventObjects  ( TRUE  );

		m_precise_branch_mispredict->setEnabled( TRUE );
		m_precise_branch_mispredict->setChecked( TRUE );
	}

	for (i = 0; i < MAX_EVENTNUM; i ++) {
		if (! evt.comboindex[i]) {
			m_checks[i][CHECK_OS]->setEnabled( FALSE );
			m_checks[i][CHECK_USR]->setEnabled( FALSE );
			m_lines[i][LINE_COUNT]->setEnabled( FALSE);
		}
	}

	onIncludeJava();

	return TRUE;
} // OptionsDlg::initialize()


void OptionsDlg::getEbpSettings( EBS_SESSION_OPTIONS * settings )
{
	int i;
	if( !ebpAllowed() ) return;

	for (i = 0; i < MAX_EVENTNUM; i ++) {
		//we route it through the event index array, in case the events were 
		//sorted alphabetically
		settings->comboindex[i] = 
			eventIndexArray[m_combos[i][EVENT_NAME]->currentItem()];

		settings->srcunit [i] = 0;
		settings->eventmask[i] = 
			m_combos[i][EVENT_TYPE]->currentText().toUInt (0, HEX_BASE);

		// counts and other stuff
		settings->count[i] = m_lines[i][LINE_COUNT]->text().toULong();
		settings->Os[i] = (m_checks[i][CHECK_OS]->isChecked() ? 1 : 0);
		settings->Usr[i] = (m_checks[i][CHECK_USR]->isChecked() ? 1 : 0);

		settings->unitmask[i] = getEventUnitMask (i);
		settings->op_name[i] = "";
	}

	EventList::const_iterator elit;
	for( elit = m_events_file.FirstEvent(); elit 
		!= m_events_file.EndOfEvents(); elit++ ) {
			for (i = 0; i < MAX_EVENTNUM; i ++) {
				if (settings->eventmask[i] == elit->value &&
					(0 != m_combos[i][EVENT_NAME]->currentItem()))
					settings->op_name[i] = elit->op_name;
			}
	}
} //OptionsDlg::GetEbpSettings


bool OptionsDlg::ebpAllowed()
{
	bool amd_machine = m_cpu_desc->text().contains("AMD");
	return (amd_machine && m_has_local_apic);
}


uint OptionsDlg::getEventUnitMask (int index)
{
	unsigned int mask = 0;

	for (int i = 0; i < CHECK_OS; i++)
		mask |= (m_checks[index][i]->isChecked() ?  1 : 0) << i;

	return mask;
}


void OptionsDlg::setEventUnitMask( uint mask, int index )
{
	for (int i = 0; i < CHECK_OS; i++)
		m_checks[index][i]->setChecked(mask & (1 << i));
}


// Event handler that gets fired when a new value is selected(highlighted)
void OptionsDlg::onEventValueChanged (int value, int index)
{
	m_combos[index][EVENT_NAME]->setCurrentItem (value);
}


void OptionsDlg::onEventChanged (int value, int index)
{
	int i;
	m_combos[index][EVENT_TYPE]->setCurrentItem (value);

	// intialized all unit mask options to unchecked reserved
	for (i = 0; i < CHECK_OS; i++) {
		m_checks[index][i]->setEnabled (FALSE);
		m_checks[index][i]->setChecked( FALSE );
		m_checks[index][i]->setText( "Reserved" );
	}	
	if( 0 == value ) {
		m_lines[index][LINE_COUNT]->setEnabled( FALSE );
		m_lines[index][LINE_SOURCE]->setEnabled( FALSE );

		m_checks[index][CHECK_OS]->setChecked( FALSE );
		m_checks[index][CHECK_OS]->setEnabled( FALSE );

		m_checks[index][CHECK_USR]->setChecked( FALSE );
		m_checks[index][CHECK_USR]->setEnabled( FALSE );

	} else {
		m_lines[index][LINE_COUNT]->setEnabled( TRUE );
		m_lines[index][LINE_SOURCE]->setEnabled( TRUE );

		m_checks[index][CHECK_OS]->setChecked( TRUE );
		m_checks[index][CHECK_OS]->setEnabled( TRUE );

		m_checks[index][CHECK_USR]->setChecked( TRUE );
		m_checks[index][CHECK_USR]->setEnabled( TRUE );

		// find the current event
		CpuEvent event;
		unsigned int temp = m_combos[index][EVENT_TYPE]->currentText().toUInt (0, HEX_BASE);

		if ( m_events_file.FindEventByValue (temp, event) ) {
			m_lines[index][LINE_SOURCE]->setText( event.source );

			UnitMaskList::const_iterator umit;
			for ( umit = m_events_file.FirstUnitMask (event); 
				umit != m_events_file.EndOfUnitMasks (event);  ++umit ) {
					m_checks[index][umit->value]->setEnabled( TRUE );
					m_checks[index][umit->value]->setText( umit->name );
					m_checks[index][umit->value]->setChecked( TRUE );
			}
		}
	}
} //OptionsDlg::onEventChanged


void OptionsDlg::onStopAppChanged()
{
	bool temp = m_stop_on_app->isChecked();
	m_no_duration->setEnabled(temp);
	if (!temp) m_no_duration->setChecked( FALSE );
}


void OptionsDlg::onNoDuration()
{
	if (m_no_duration->isChecked()) {
		m_last_duration = m_duration->text();
		m_duration->setText ("0");
		m_duration->setEnabled (FALSE);
		m_break_after->setEnabled (FALSE);
		m_break_after->setChecked (FALSE);
	} else {
		if ("0" != m_last_duration)
			m_duration->setText (m_last_duration);
		else
			m_duration->setText ("10");
		m_duration->setEnabled(TRUE);
		m_break_after->setEnabled (TRUE);
	}
}

void OptionsDlg::setOptions(QString session, QString file, int interval, 
							int duration, int delay, QString caption)
{
	m_session_name->setText(session);
	m_file_name->setText(file);

	QString tmpStr;
	tmpStr.sprintf("%d", interval);
	m_interval->setText(tmpStr);

	tmpStr.sprintf("%d", duration);
	m_duration->setText(tmpStr);

	tmpStr.sprintf("%d", delay);
	m_delay->setText(tmpStr);

	setCaption(caption);
}


//This will validate that no two counters are set to the same event
//returns whether the events are valid or not
//This sets the focus on the errant event, if applicable
bool OptionsDlg::validateEvents()
{
	int temp;
	if (m_event_trigger->isChecked()) {
		// make sure no duplicated events
		for (int i= 0; i < MAX_EVENTNUM; i++) {
			temp =  m_combos[i][EVENT_NAME]->currentItem();
			if (0 != temp) {
				for (int j = i + 1; j < MAX_EVENTNUM; j++) {
					if (m_combos[j][EVENT_NAME]->currentItem() == temp) {
						m_tab_widget->setCurrentPage( EVENT_PAGE );
						m_events_widget->setCurrentPage (j);
						m_combos[j][EVENT_TYPE]->setFocus( );
						QMessageBox::information (this, "Invalid Input", 
							"This event is already being counted." );
						return FALSE;
					}
				}
				if (0 == m_lines[i][LINE_COUNT]->text().toUInt()) {
					m_tab_widget->setCurrentPage( EVENT_PAGE );
					m_events_widget->setCurrentPage (i);
					m_lines[i][LINE_COUNT]->setFocus();
					QMessageBox::information (this, "Invalid Input", "Event "
						"count must be more than 0");
					return FALSE;
				}
			}
		}
	}
	return TRUE;
} //OptionsDlg::validateEvents()


ATuneFile * OptionsDlg::getCawFile()
{
	return m_caw_file;
}

//adapted from the code from ioptionsdlg.ui.cpp, generated from the original
// ui layout.  We adapted it to avoid a whole lot of repeated code.
bool OptionsDlg::createEvents()
{
	int i, j;
	m_events_widget = new QTabWidget( tab_2 );

	for (i = 0; i < MAX_EVENTNUM; i++) {
		QSpacerItem * spacers[SPACER_COUNT];
		QLabel * labels[LABEL_USR + 1];
		QHBoxLayout * h_layouts[3];
		QGridLayout * grid_layouts[8];
		QGroupBox * unit_mask_box;

		m_event_tabs[i] = new QWidget( m_events_widget);
		grid_layouts[0] = new QGridLayout( m_event_tabs[i], 1, 1, 11, 6);
		h_layouts[0] = new QHBoxLayout( 0, 0, 6);

		labels[LABEL_EVENT] = new QLabel( tr( "Event:"), m_event_tabs[i]);
		labels[LABEL_EVENT]->setMaximumSize( QSize( 40, 21 ) );
		h_layouts[0]->addWidget( labels[LABEL_EVENT] );

		m_combos[i][EVENT_TYPE] = new ElementComboBox (FALSE, m_event_tabs[i]);
		m_combos[i][EVENT_TYPE]->setMaximumSize( QSize( 60, 21 ) );
		m_combos[i][EVENT_TYPE]->setIndex (i);
		h_layouts[0]->addWidget( m_combos[i][EVENT_TYPE] );
		QObject::connect ( m_combos[i][EVENT_TYPE], 
			SIGNAL (valueChanged ( int, int )), 
			SLOT (onEventValueChanged (int, int)) );
		QObject::connect (this, SIGNAL (setChildUpdateEnable (bool)),
			m_combos[i][EVENT_TYPE],
			SLOT (setUpdatesEnabled (bool)));

		m_combos[i][EVENT_NAME] = new ElementComboBox (FALSE, m_event_tabs[i]);
		m_combos[i][EVENT_NAME]->setIndex (i);
		h_layouts[0]->addWidget( m_combos[i][EVENT_NAME] );
		QObject::connect ( m_combos[i][EVENT_NAME], 
			SIGNAL (valueChanged ( int, int )), 
			SLOT (onEventChanged (int, int)) );
		QObject::connect (this, SIGNAL (setChildUpdateEnable (bool)),
			m_combos[i][EVENT_NAME],
			SLOT (setUpdatesEnabled (bool)));

		grid_layouts[0]->addMultiCellLayout( h_layouts[0], 0, 0, 0, 1 );

		unit_mask_box = new QGroupBox( m_event_tabs[i]);
		unit_mask_box->setColumnLayout(0, Qt::Vertical );
		unit_mask_box->layout()->setSpacing( 6 );
		unit_mask_box->layout()->setMargin( 11 );

		grid_layouts[1] = new QGridLayout( unit_mask_box->layout() );
		grid_layouts[1]->setAlignment( Qt::AlignTop );

		for (j = 0; j < CHECK_OS; j++) {
			m_checks[i][j] = new QCheckBox( unit_mask_box);
			m_checks[i][j]->setEnabled( FALSE );
			QObject::connect (this, SIGNAL (setChildUpdateEnable (bool)),
				m_checks[i][j], SLOT (setUpdatesEnabled (bool)));
			grid_layouts[1]->addWidget( m_checks[i][j], j, 0 );
		}

		spacers[0] = new QSpacerItem( 220, 21, QSizePolicy::Expanding, 
			QSizePolicy::Minimum );
		grid_layouts[1]->addItem( spacers[0], 8, 0 );

		spacers[1] = new QSpacerItem( 21, 210, QSizePolicy::Minimum, 
			QSizePolicy::Expanding );
		grid_layouts[1]->addMultiCell( spacers[1], 0, 7, 1, 1 );
		grid_layouts[0]->addWidget( unit_mask_box, 1, 1 );

		grid_layouts[2] = new QGridLayout( 0, 1, 1, 0, 6); 
		grid_layouts[3] = new QGridLayout( 0, 1, 1, 0, 6); 
		spacers[2] = new QSpacerItem( 49, 30, QSizePolicy::Expanding, 
			QSizePolicy::Minimum );
		grid_layouts[3]->addItem( spacers[2], 1, 1 );

		grid_layouts[4] = new QGridLayout( 0, 1, 1, 0, 6);
		labels[LABEL_SOURCE] = new QLabel( tr( "Src:" ), m_event_tabs[i]);
		grid_layouts[4]->addWidget( labels[LABEL_SOURCE], 1, 0 );

		grid_layouts[5] = new QGridLayout( 0, 1, 1, 0, 6);
		m_lines[i][LINE_SOURCE] = new QLineEdit( m_event_tabs[i]);
		m_lines[i][LINE_SOURCE]->setReadOnly( TRUE );
		m_lines[i][LINE_SOURCE]->setMaximumSize( QSize( 38, 22 ) );
		grid_layouts[5]->addWidget( m_lines[i][LINE_SOURCE], 1, 0);

		spacers[3] = new QSpacerItem( 20, 20, QSizePolicy::Expanding, 
			QSizePolicy::Minimum );
		grid_layouts[5]->addItem( spacers[3], 1, 1 );

		m_lines[i][LINE_COUNT] = new QLineEdit( "100000", m_event_tabs[i]);
		m_lines[i][LINE_COUNT]->setMaximumSize( QSize( 100, 22 ) );
		grid_layouts[5]->addMultiCellWidget (m_lines[i][LINE_COUNT], 0, 0, 0, 
			1);

		grid_layouts[4]->addMultiCellLayout( grid_layouts[5], 0, 1, 1, 1 );

		labels[LABEL_COUNT] = new QLabel( tr( "Count:"), m_event_tabs[i]);
		grid_layouts[4]->addWidget( labels[LABEL_COUNT], 0, 0 );
		grid_layouts[3]->addMultiCellLayout( grid_layouts[4], 0, 0, 0, 1 );

		grid_layouts[6] = new QGridLayout( 0, 1, 1, 0, 6);
		h_layouts[1] = new QHBoxLayout( 0, 0, 6);

		labels[LABEL_OS] = new QLabel( tr( "Os" ), m_event_tabs[i] );
		h_layouts[1]->addWidget( labels[LABEL_OS] );

		labels[LABEL_USR] = new QLabel( tr( "Usr" ), m_event_tabs[i] );
		h_layouts[1]->addWidget( labels[LABEL_USR]);

		grid_layouts[6]->addLayout( h_layouts[1], 0, 0 );
		h_layouts[2] = new QHBoxLayout( 0, 0, 6);

		m_checks[i][CHECK_OS] = new QCheckBox( m_event_tabs[i] );
		m_checks[i][CHECK_OS]->setChecked( TRUE );
		QObject::connect (this, SIGNAL (setChildUpdateEnable (bool)),
			m_checks[i][CHECK_OS], SLOT (setUpdatesEnabled (bool)));
		h_layouts[2]->addWidget( m_checks[i][CHECK_OS] );

		m_checks[i][CHECK_USR] = new QCheckBox( m_event_tabs[i] );
		m_checks[i][CHECK_USR]->setChecked( TRUE );
		QObject::connect (this, SIGNAL (setChildUpdateEnable (bool)),
			m_checks[i][CHECK_USR], SLOT (setUpdatesEnabled (bool)));
		h_layouts[2]->addWidget( m_checks[i][CHECK_USR] );

		grid_layouts[6]->addLayout( h_layouts[2], 1, 0 );
		grid_layouts[3]->addLayout( grid_layouts[6], 1, 0 );
		grid_layouts[2]->addLayout( grid_layouts[3], 0, 0 );

		spacers[4] = new QSpacerItem( 110, 60, QSizePolicy::Minimum, 
			QSizePolicy::Expanding );
		grid_layouts[2]->addItem( spacers[4], 1, 0 );
		spacers[5] = new QSpacerItem( 20, 50, QSizePolicy::Expanding, 
			QSizePolicy::Minimum );
		grid_layouts[2]->addItem( spacers[5], 0, 1 );

		grid_layouts[0]->addLayout( grid_layouts[2], 1, 0 );

		QString temp;
		temp.sprintf (tr("Counter %d"), i);
		m_events_widget->insertTab( m_event_tabs[i], temp );

		unit_mask_box->setTitle( tr( "Unit Masks" ) );
		for (j = 0; j < CHECK_OS; j++) {
			m_checks[i][j]->setText( tr( "Reserved" ) );
		}
	}
	tabLayout_2->addWidget( m_events_widget, 1, 0 );
	return true;
} //OptionsDlg::createEvents

int OptionsDlg::indexOfEvent (int event)
{
	int ret_value = 0;
	for (int i = 0; i < m_combos[0][EVENT_TYPE]->count(); i++) {
		if (eventIndexArray[i] == event) {
			ret_value = i;
			break;
		}
	}
	return ret_value;
}

//swaps the strings in all event tabs and the order of the indexes
void OptionsDlg::swapEvents (int a, int b)
{
	if (a ==b )
		return;
	QString old_type = m_combos[0][EVENT_TYPE]->text (a);
	QString new_type = m_combos[0][EVENT_TYPE]->text (b);
	QString old_name = m_combos[0][EVENT_NAME]->text (a);
	QString new_name = m_combos[0][EVENT_NAME]->text (b);
	int temp = eventIndexArray[b];

	eventIndexArray[b] = eventIndexArray[a];
	eventIndexArray[a] = temp;

	for (int i=0; i < MAX_EVENTNUM; i++) {
		m_combos[i][EVENT_TYPE]->changeItem ( new_type, a);
		m_combos[i][EVENT_TYPE]->changeItem ( old_type, b);
		m_combos[i][EVENT_NAME]->changeItem ( new_name, a);
		m_combos[i][EVENT_NAME]->changeItem ( old_name, b);
	}
}


//returns "< 0" if a < b, "== 0" if b == a, and "> 0" if a > b
inline int OptionsDlg::compareEvents (ElementComboBox * sort_by, int a, int b)
{
	return sort_by->text (a).compare (sort_by->text (b));
}


void OptionsDlg::quickSortEvents (ElementComboBox * sort_by, int low, int high)
{
	int temp_l = low;
	int temp_h = high;
	if (low >= high) 
		return;
	else if ((low == (high -1)) && (compareEvents (sort_by, low, high)) > 0){
		swapEvents (low, high);
		return;
	}

	swapEvents (high, ((low + high) / 2) );
	while (temp_l < temp_h) {
		while ((temp_l < temp_h) && (compareEvents (sort_by, temp_l, high) <= 0))
			temp_l++;
		while ((temp_l < temp_h) && (compareEvents (sort_by, high, temp_h) <= 0))
			temp_h--;
		if (temp_l < temp_h) {
			swapEvents (temp_l, temp_h);
		}
	}
	swapEvents (high, temp_h);
	quickSortEvents (sort_by, low, (temp_l - 1));
	quickSortEvents (sort_by, (temp_h + 1), high);
	return;
}


ElementComboBox::ElementComboBox (bool rw, QWidget * parent, const char * name)
:QComboBox (rw, parent, name) 
{
	QObject::connect (this, SIGNAL (highlighted (int)), this, 
		SLOT (onEventChanged (int)));
}


ElementComboBox::~ElementComboBox()
{
}


void ElementComboBox::setIndex ( int index)
{
	m_index = index;
}


void ElementComboBox::onEventChanged ( int value )
{
	emit valueChanged (value, m_index);
}
