//$Id: optionsdlg.h,v 1.5 2006/08/17 14:25:23 ssuthiku Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/


#ifndef OPTIONSDLG_H
#define OPTIONSDLG_H

#include <qvalidator.h>
#include <qcombobox.h>
#include <qradiobutton.h>

//generated from ioptionsdlg.ui
#include "ioptionsdlg.h"

#include "stdafx.h"
#include "eventsfile.h"
#include "atunefile.h"
#include "projectfile.h"

enum OptionsDlgLocalEnum 
{
	GENERAL_PAGE = 0,
	TIMER_PAGE = 1,
	EVENT_PAGE = 2,
	PIV_FAMILY = 15,
	PIII_FAMILY = 6,
	SORT_BY_INDEX = 0
};

class ElementComboBox : public QComboBox
{
	Q_OBJECT
public:
	ElementComboBox (bool rw, QWidget * parent = 0, const char * name = 0);
	~ElementComboBox();
	void setIndex (int index);

signals:
	void valueChanged ( int value, int index );

	private slots:
		void onEventChanged ( int value );

private:
	int m_index;
};


class OptionsDlg : public IOptionsDlg
{ 
	Q_OBJECT

public:
	OptionsDlg (bool new_file, ATuneFile * caw_file, QWidget* parent = 0, 
		const char* name = 0, bool modal = FALSE, 
		bool warning = FALSE, WFlags fl = WStyle_Customize 
		| WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu |
		WDestructiveClose);
	~OptionsDlg();
	void setOptions(QString session, QString file, int interval, int duration,
		int delay, QString caption);

	bool initialize();
	ATuneFile * getCawFile();

signals:
	void setChildUpdateEnable ( bool enabled);

	public slots:
		// Overrides from IOptionsDlg
		void onApply();
		void onOk();
		void onBrowseDir();
		void onBrowsePrograms();
		void onBrowseWorkDir();

		void onProgramChanged ( const QString &txt );

		void onEventChanged (int value, int index);

		void onEventValueChanged (int value, int index);
		void onSortEvents ();

		void onTriggerChanged();
		void enableEventObjects ( bool enabled );
		void enableTimerObjects ( bool enabled );

		void onStopAppChanged();
		void onNoDuration();
		void onIncludeJava();
private:
	bool ebpAllowed();
	void getEbpSettings ( EBS_SESSION_OPTIONS *settings );
	void loadAMDEvents (const char * path);
	void detectCpu();
	bool saveChanges();
	bool validate();
	bool validateEvents();

	uint getEventUnitMask (int index);
	void setEventUnitMask (uint mask, int index);

	int * eventIndexArray;
	int indexOfEvent (int event);
	void swapEvents (int a, int b);
	int compareEvents (ElementComboBox * sort_by, int a, int b);
	void quickSortEvents (ElementComboBox * sort_by, int low, int high);
	QString getCpuModelName();

	bool m_new;
	bool m_show_apic_warning;

	QDoubleValidator * m_val_interval;
	QIntValidator * m_val_duration;
	QIntValidator * m_val_delay;
	ATuneFile * m_caw_file;

	QString m_default_project_dir;
	bool m_has_local_apic;
	CEventsFile m_events_file;
	QString m_last_duration;

	//related to event tab creation
	bool createEvents();
	QTabWidget * m_events_widget;
	QWidget * m_event_tabs[MAX_EVENTNUM];
	QLineEdit * m_lines[MAX_EVENTNUM][LINE_SOURCE + 1];
	QCheckBox * m_checks[MAX_EVENTNUM][CHECK_USR + 1];
	ElementComboBox * m_combos[MAX_EVENTNUM][EVENT_TYPE + 1];
};

#endif // OPTIONSDLG_H
