//$Id: oprofile_interface.h,v 1.2 2006/05/15 22:09:45 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef OPROFILE_INTERFACE_H_
#define OPROFILE_INTERFACE_H_

#include "stdafx.h"
#include <qstring.h>
#include <qthread.h>
#include <qobject.h>
#include <qprocess.h>
#include "oprofile_interface.h"


/**
 * Interface between CodeAnalyst and Oprofile. 
 * The class essentially does the same thing as opcontrol. 
 *
 * Note: The function names do not follow the QT coding
 * standard to reflect the equivalent name in opcontrol 
 * script.
 */
class oprofile_interface:QObject {
    Q_OBJECT
public:
    oprofile_interface ();
    bool load_module ();
    void do_init ();
    bool do_start_daemon (bool novmlinux, QString & vmlinux_dir,
                          unsigned int buffer_size, unsigned int note_size,
                          bool anon_profile);
    bool do_restart_daemon (bool novmlinux, QString & vmlinux_dir, bool
    anon_profile);
    void set_op_events (op_event_property * properties);
    void do_dump ();
    void do_kill_daemon ();


    QString get_kernel_range ();
    QString & get_std_err();
    
    static void do_reset ();

private:
    bool load_module_24 ();
    bool load_module_26 ();
    bool op_dev_exist ();
    bool help_start_daemon (bool novmlinux, QString & vmlinux_dir, bool anon_profile);
    bool file_contains (const char *file, const char *str,
                        QString * line = NULL);
    int n_file_contains (const char *file, const char *str, QString * line);

    bool do_setup ();

    void rm_devices_24 ();

    int create_devices_24 ();

    int mount_opfs ();

    int insert_op_module ();

    int create_devices (const char *dev, int major_nr, int minor_nr);

    int exec_command (QStringList & args);

    int exec_command_no_wait (const char *command, const char *args);

    bool do_sysctl_setup (unsigned int buffer_size, unsigned int note_size);

    void customEvent (QCustomEvent * pCustEvent);

    void generate_op_event_arg (QString & arg);

    void do_dump_stop ();

    void set_ctr_param (int event, QString field, int value);

    void set_param (QString field, int value);

private:
    bool m_KernelSupport;

    op_event_property m_EventProperties[MAX_EVENTNUM];
    
    QProcess * m_proc;

    QString m_StdErr;

};

#endif
