#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <signal.h>
#include <popt.h>
#include <string.h>
#include <sys/wait.h>

#include "config.h"

#define LOCK_PATH "/var/lib/oprofile/lock"
#define SAMPLES_CURRENT_PATH "/var/lib/oprofile/samples/current/"

int sigterm = 0;
int sigkill = 0;
int rmlock = 0;

static struct poptOption options[] = {
{ "term"  , 't', POPT_ARG_NONE, &sigterm, 0, "Send SIGTERM to oprofiled", NULL },
{ "term"  , 'k', POPT_ARG_NONE, &sigkill, 0, "Send SIGKILL to oprofiled", NULL },
{ "rmlock", 'r', POPT_ARG_NONE, &rmlock , 0, "Remove Lock file Only"    , NULL }, 
POPT_AUTOHELP
{ NULL, 0, 0, NULL, 0, NULL, NULL },
};


int do_options(int argc, char const * argv[])
{
	poptContext optcon;
	int c = 0, ret = 0;

	optcon = poptGetContext(NULL, argc, argv, options, 0);

	if (argc < 2) {
		poptPrintUsage(optcon, stdout, 0);
		ret = -1;
		goto out;
	}

	while (c > -1) 
		c = poptGetNextOpt(optcon);

	poptFreeContext(optcon);
out:
	return ret;
}

int get_pid_from_lock_file()
{
	FILE * lock_file;
	struct stat st;
	char buf[64];
	int ret = -1;
	int size = 0;

	// Use /var/lib/oprofile/lock
	if (0 != stat(LOCK_PATH, &st))
	{
		return ret;	
	}
	if (!(lock_file = fopen(LOCK_PATH, "r"))) {
		return ret;	
	}

	if((size = fread(&buf, 1, sizeof(buf), lock_file)) == 0)
		goto out;
	else	
		buf[size+1] = 0;

	ret = atol(buf);
out:
	return ret;
}


int get_pid_from_pidof()
{
	pid_t pid;
	int pipe_fd[2];
	int status = 0;
	int readCnt = 0;
	char buf[10];

	char *cmd_pidof[3];

	// Command
	cmd_pidof[0] = PIDOF;
	cmd_pidof[1] = "oprofiled";
	cmd_pidof[2] = NULL;
	

	// Pipe
	if(pipe(pipe_fd) < 0)
		return -1;

	//Create Process
	if( (pid = fork()) < 0 ) {
		return -1;
	} else if ( pid == 0 ) { // child
		// Setup stdout
		close (pipe_fd[0]);
		dup2 (pipe_fd[1],1);
		close (pipe_fd[1]);
	
		// Execute
		execvp(cmd_pidof[0], cmd_pidof);
	
	} else { //parent
		// Setup stdin
		close (pipe_fd[1]);
		dup2 (pipe_fd[0],0);
		close (pipe_fd[0]);

		// Read from pipe
		readCnt = read(0, buf,10);
		waitpid (pid, &status, 0);
	}
	
	if(readCnt <= 0)
		return -1;
	
	return atoi(buf);
}

int remove_lock()
{
	int ret = -1;
	pid_t pid;

	if((pid = get_pid_from_pidof()) <= 0) {
		fprintf(stderr,
			"killoprofiled: Cannot remove lock file since oprofiled is still running.\n");
	} else {
		ret = 0;
		unlink(LOCK_PATH);	
	}
	return ret;
}

int send_signal(void )
{
	int ret = -1;
	pid_t pid;
 
	// Suravee commented out.
	// NOTE: This approach is a security risk 
	// since some might update /var/lib/oprofile/lock
	// to be any number
	//if((pid = get_pid_from_lock_file()) == -1)
	if((pid = get_pid_from_pidof()) <= 0) {
		fprintf(stderr,
			"killoprofiled: Could not locate oprofiled.\n");
		goto out;
	}

	if (sigterm != 0) {
		printf("killoprofiled: sending SIGTERM to pid %d\n", pid);
		if((ret = kill(pid, SIGTERM)) != 0)
			fprintf(stderr,
			"killoprofiled: Error sending SIGTERM to oprofiled.\n");
	} else if (sigkill != 0) {
		printf("killoprofiled: sending SIGKILL to pid %d\n", pid);
		if((ret = kill(pid, SIGKILL)) != 0)
			fprintf(stderr,
			"killoprofiled: Error sending SIGKILL to oprofiled.\n");
		else
			unlink(LOCK_PATH);
	}
out:
	return ret;
}

int main(int argc, char const * argv[]) 
{
	int ret = -1;
	
	if(0 != getuid()){
		fprintf(stderr,"killoprofiled: Error, must be root\n");
		goto out;
	}

	if (0 > (ret = do_options(argc, argv)))
		goto out;

	// Remove lock only. Not kill daemon
	if(rmlock) {
		ret = remove_lock();
		goto out;
	}

	// Sending signal to kill daemon
	ret = send_signal();
out:
	return ret;
}
